{% extends "digitizer/insertable.j2" -%}
{% block detail -%}
{% if o.policy -%}
/gate/digitizer/{{ o.base.name }}/{{ o.name }}/setPolicy {{ o.policy }}
{% endif -%}
/gate/digitizer/{{ o.base.name }}/{{ o.name }}/setDepth  {{ o.depth }}
{%- endblock -%}