{% from "source/source.j2" import source -%}
{% macro source_list(o) -%}
{% for s in o.sources -%}
{{ source(s) }}
{% endfor -%}
/gate/source/list
{% endmacro -%}
{% if o -%}
{{ source_list(o) }}
{%- endif %}