{% extends "geometry/volume/volume.j2" %}
{% block detail -%}
/gate/{{ o.name }}/geometry/setRmin         {{ o.rmin }} {{ o.unit }} 
/gate/{{ o.name }}/geometry/setRmax         {{ o.rmax }} {{ o.unit }}
/gate/{{ o.name }}/geometry/setPhiStart         {{ o.phi_start }} {{ o.unit }}
/gate/{{ o.name }}/geometry/setDeltaPhi         {{ o.delta_phi }} {{ o.unit }}
/gate/{{ o.name }}/geometry/setThetaStart         {{ o.theta_start }} {{ o.unit }}
/gate/{{ o.name }}/geometry/setDeltaTheta         {{ o.delta_theta }} {{ o.unit }}
{% endblock detail -%}

