{% extends "geometry/volume/repeater/repeater.j2" %}
{% from "utils.j2" import vec3 %}
{% block detail -%}
/gate/{{ o.volume.name }}/cubicArray/setRepeatNumberX        {{ o.scale.x }}  
/gate/{{ o.volume.name }}/cubicArray/setRepeatNumberY        {{ o.scale.y }}  
/gate/{{ o.volume.name }}/cubicArray/setRepeatNumberZ        {{ o.scale.z }}  
/gate/{{ o.volume.name }}/cubicArray/setRepeatVector         {{ vec3(o.rv) }} 
{% endblock detail -%}

