{% macro insertable(o) -%}
{% if o.is_define_name -%}
/gate/digitizer/name {{ o.name }}
{% endif -%}   
{% if o.is_explict_insert -%}
/gate/digitizer/{{ [o.base.name, '/'] | join if o.base }}insert {{ o.insertable_type }}
{% endif -%}
{% block detail scoped -%}
{%- endblock -%}
{% endmacro -%}
{% if o -%}
{{ insertable(o) }}
{%- endif %}