{% macro repeater(o) -%}
/gate/{{ o.volume.name }}/repeaters/insert          {{ o.repeater_type }}
{% block detail scoped -%}
{%- endblock -%}
{%- endmacro -%}
{% if o -%}
{{ repeater(o) }}
{%- endif %}