{% extends "geometry/volume/volume.j2" %}
{% block detail -%}
/gate/{{ o.name }}/geometry/setXLength         {{ o.size.x }} {{ o.size.unit }}
/gate/{{ o.name }}/geometry/setYLength         {{ o.size.y }} {{ o.size.unit }}
/gate/{{ o.name }}/geometry/setZLength         {{ o.size.z }} {{ o.size.unit }}
{% endblock detail -%}
