{% from "physics/process.j2" import process -%}
{% from "physics/cuts.j2" import cuts -%}
{% macro physics(o) -%}
{% for p in o.pl -%}
{{ process(p) }}
{% endfor -%}
/gate/physics/processList Enabled
/gate/physics/processList Initialized
{% for c in o.cl -%}
{{ cuts(c) }}
{% endfor -%}
{% endmacro -%}
{% if o -%}
{{ physics(o) }}
{%- endif %}