{% extends "source/shape/shape.j2" %}
{% block detail -%}
/gate/source/{{ o.src.name }}/gps/radius0 {{ o.radius0 }} mm 
/gate/source/{{ o.src.name }}/gps/radius {{ o.radius }} mm 
{% endblock detail -%}