{% macro cuts(o) -%}
{% for p in o.cuts -%}
/gate/physics/{{ p }}/SetCutInRegion      {{ o.volume.name }} {{ o.cuts[p] }} mm
{% endfor -%}
{% if o.max_step -%}
/gate/physics/SetMaxStepSizeInRegion    {{ o.volume.name }} {{ o.max_step }} mm
{% endif -%}
{% endmacro -%}
{% if o -%}
{{ cuts(o) }}
{%- endif %}