{% macro vis(name, force_fire_frame, color) -%}
{% if force_fire_frame -%}
/gate/{{name}}/vis/forceWireframe
{% endif -%}
{% if color -%}
/gate/{{name}}/vis/setColor             {{ color }}
{% endif -%}
{% endmacro -%}
{% macro vec3(o) -%} {{ o.x }} {{ o.y }} {{o.z}}{{ ' '+o.unit if not o.unit is none }} {%- endmacro -%}