{% extends "source/shape/shape.j2" %}
{% block detail -%}
/gate/source/{{ o.src.name }}/gps/halfx {{ o.half_size[0] }} mm 
/gate/source/{{ o.src.name }}/gps/halfy {{ o.half_size[1] }} mm 
{% endblock detail -%}