from .api import *
"""
======
Pygate 
======

--------------
Pre simulation
--------------

Mac generator
=============

python API
----------

Low level API
^^^^^^^^^^^^^

High level API
^^^^^^^^^^^^^^

CLI
---

. code :: shell
    pygate generate mac --script experiment.py
    pygate generate mac --config experiment.yml
    pygate generate mac --config experiment.json


Shell script generator
======================


"""