{% extends "digitizer/insertable.j2" -%}
{% block detail -%}
/gate/digitizer/{{ o.name }}/addInputName {{ o.input1.name }}
/gate/digitizer/{{ o.name }}/addInputName {{ o.input2.name }}
/gate/digitizer/{{ o.name }}/usePriority {{ o.use_priority | lower }}
{%- endblock -%}